@@ -1,3 +1,4 @@
+from operator import truediv
 from typing import List, Dict, Tuple
 from itertools import combinations_with_replacement
 import numpy as np
@@ -40,23 +41,10 @@ def create_face(block:Block,imin:int,imax:int,jmin:int,jmax:int,kmin:int,kmax:in
                 f.add_vertex(block.X[i,j,kmin], block.Y[i,j,kmin], block.Z[i,j,kmin],i,j,kmin)
     return f
 
-# def save_face(face_list:List[Face],block_indices:List[int]):
-#     """Converts a list of faces and block indices to a dictionary 
-
-#     Args:
-#         face_list (List[Face]): [description]
-#         block_indices (List[int]): [description]
-
-#     Returns:
-#         (List[dict]): Dictionary specifying the properties of the face 
-#     """
-#     temp = list()
-#     for i,f in enumerate(face_list):
-#         temp.append({'block_indx':block_indices[i], 'IMIN':f.IMIN, 'IMAX':f.IMAX, 'JMIN':f.JMIN, 'JMAX':f.JMAX,'KMIN':f.KMIN, 'KMAX':f.KMAX}) 
-#     return temp 
 
 def periodicity_fast(blocks:List[Block],outer_faces:List[Face], matched_faces:List[Dict[str,int]], periodic_direction:str='k', rotation_axis:str='x',nblades:int=55):
-    """Reduces the size of the blocks by a factor of the minimum gcd. This speeds up finding the connectivity 
+    """This function is used to match a non-rotated set of blocks. 
+        Reduces the size of the blocks by a factor of the minimum gcd. This speeds up finding the connectivity 
 
     Args:
         blocks (List[Block]): List of blocks that will be scanned for perodicity
@@ -81,6 +69,7 @@ def periodicity_fast(blocks:List[Block],outer_faces:List[Face], matched_faces:Li
         block = blocks[block_indx]
         gcd_array.append(math.gcd(block.IMAX-1, math.gcd(block.JMAX-1, block.KMAX-1)))
     gcd_to_use = min(gcd_array) # You need to use the minimum gcd otherwise 1 block may not exactly match the next block. They all have to be scaled the same way.
+    print(f"gcd to use {gcd_to_use}")
     new_blocks = reduce_blocks(deepcopy(blocks),gcd_to_use)
     # Reduce face matches for the block 
     for i in range(len(matched_faces)):
@@ -149,7 +138,34 @@ def periodicity_fast(blocks:List[Block],outer_faces:List[Face], matched_faces:Li
         
     return periodic_faces_export, outer_faces_export, periodic_faces, outer_faces_all
 
-def periodicity(blocks:List[Block],outer_faces:List[Face], matched_faces:List[Dict[str,int]], periodic_direction:str='k', rotation_axis:str='x',nblades:int=55):
+def create_rotation_matrix(rotation_angle:float, rotation_axis:str="x"):
+    """Creates a rotation matrix given an angle and axis 
+
+    Args:
+        rotation_angle (float): Rotation angle in radians
+        rotation_axis (str, optional): Axis of rotation "x", "y", or "z". Defaults to "x".
+
+    Returns:
+        np.ndarray: 3x3 rotation matrix 
+    """
+    
+    if rotation_axis=='x':
+        rotation_matrix = np.array([[1,0,0],
+                            [0,cos(rotation_angle),-sin(rotation_angle)],
+                            [0,sin(rotation_angle),cos(rotation_angle)]])
+
+    elif rotation_axis=='y':
+        rotation_matrix = np.array([[cos(rotation_angle),0,sin(rotation_angle)],
+                            [0,1,0],
+                            [-sin(rotation_angle),0,cos(rotation_angle)]])
+    elif rotation_axis=='z':
+        rotation_matrix = np.array([[cos(rotation_angle),-sin(rotation_angle), 0],
+                            [sin(rotation_angle),cos(rotation_angle), 0],
+                            [0, 0, 1]])
+
+    return rotation_matrix 
+
+def periodicity(blocks:List[Block],outer_faces:List[Dict[str,int]], matched_faces:List[Dict[str,int]], periodic_direction:str='k', rotation_axis:str='x',nblades:int=55):
     """This function is used to check for periodicity of the other faces rotated about an axis 
         The way it works is to find faces of a constant i,j, or k value
 
@@ -172,28 +188,9 @@ def periodicity(blocks:List[Block],outer_faces:List[Face], matched_faces:List[Di
     """
     
     rotation_angle = radians(360.0/nblades)
-    if rotation_axis=='x':
-        rotation_matrix1 = np.array([[1,0,0],
-                            [0,cos(rotation_angle),-sin(rotation_angle)],
-                            [0,sin(rotation_angle),cos(rotation_angle)]])
-        rotation_matrix2 = np.array([[1,0,0],
-                            [0,cos(-rotation_angle),-sin(-rotation_angle)],
-                            [0,sin(-rotation_angle),cos(-rotation_angle)]])
-    elif rotation_axis=='y':
-        rotation_matrix1 = np.array([[cos(rotation_angle),0,sin(rotation_angle)],
-                            [0,1,0],
-                            [-sin(rotation_angle),0,cos(rotation_angle)]])
-        rotation_matrix2 = np.array([[cos(-rotation_angle),0,sin(-rotation_angle)],
-                            [0,1,0],
-                            [-sin(-rotation_angle),0,cos(-rotation_angle)]])
-    elif rotation_axis=='z':
-        rotation_matrix1 = np.array([[cos(rotation_angle),-sin(rotation_angle), 0],
-                            [sin(rotation_angle),cos(rotation_angle), 0],
-                            [0, 0, 1]])
-
-        rotation_matrix2 = np.array([[cos(-rotation_angle),-sin(-rotation_angle), 0],
-                            [sin(-rotation_angle),cos(-rotation_angle), 0],
-                            [0, 0, 1]])
+    rotation_matrix1 = create_rotation_matrix(rotation_angle,rotation_axis)
+    rotation_matrix2 = create_rotation_matrix(-rotation_angle,rotation_axis)
+    
     # Check periodic within a block 
     periodic_found = True
     
@@ -215,12 +212,6 @@ def periodicity(blocks:List[Block],outer_faces:List[Face], matched_faces:List[Di
         face2.set_block_index(m['block2']['block_index'])
         matched_faces_all.append(face1)
         matched_faces_all.append(face2)
-    # Debug
-    # debug_per1 = create_face(blocks[2], 0, 40, 0, 100,0,0) 
-    # debug_per1.set_block_index(2)
-    # debug_per2 = create_face(blocks[2], 0, 40, 0, 100,52,52) 
-    # debug_per2.set_block_index(2)
-    # End debug 
 
     split_faces = list()         # List of split but free surfaces, this will be appended to outer_faces_to_remove list
     while periodic_found:
@@ -378,7 +369,194 @@ def periodicity(blocks:List[Block],outer_faces:List[Face], matched_faces:List[Di
         outer_faces_export.append(o.to_dict())
                         
     return periodic_faces_export, outer_faces_export, periodic_faces, outer_faces_all
-                        
+
+
+def rotated_periodicity(blocks:List[Block], matched_faces:List[Dict[str,int]], outer_faces:List[Dict[str,int]], rotation_angle:float, rotation_axis:str = "x"):
+    """Finds the peridocity/connectivity by rotating a block. This is a bit different from "periodicity" where you specify the periodic direction. 
+        This method doesn't care about the direction as long as the angle you specify results in a match between the Left Face and the Right Face         
+
+    Example 1:              
+        L      RL           R
+        | blk1 || Copy blk1 |
+        | blk2 || Copy blk2 |
+        | blk3 || Copy blk3 |
+        Rotates the set of blocks by an angle and checks the matching surfaces for R and L. 
+
+    Args:
+        blocks (List[Block]): List of blocks for a particular geometry. Do not duplicate the geometry and pass it in! 
+        outer_faces (List[Dict[str,int]]): List of outer faces in dictionary form
+        rotation_angle (float): rotation angle in between blades in degrees. factor in an additional blade 
+        rotation_axis (str, Optional): "x", "y", or "z" 
+
+    
+    periodic_faces, outer_faces_export, _, _ = rotated_periodicity(blocks,face_matches, outer_faces, rotation_angle=rotation_angle, rotation_axis = "x")
+    
+    Replaces:
+        Is the same as 
+
+        periodic_surfaces, outer_faces_to_keep,periodic_faces,outer_faces = periodicity_fast(blocks,outer_faces,face_matches,periodic_direction='k',rotation_axis='x',nblades=55)
+        and
+        periodic_surfaces, outer_faces_to_keep,periodic_faces,outer_faces = periodicity(blocks,outer_faces,face_matches,periodic_direction='k',rotation_axis='x',nblades=55)
+
+
+    Returns:
+        (Tuple): containing
+            
+            - **periodic_faces_export** (List[Dict[str,int]]):  This is list of all the surfaces/faces that match when rotated by an angle formatted as a dictionary.
+            - **outer_faces_export** (List[Dict[str,int]]): These are the list of outer faces that are not periodic formatted as a dictionary.
+            - **periodic_faces** (List[Tuple[Face,Face]]): - This is a list of Face objects that are connected to each other organized as a list of tuples: [Face1, Face2] where Face 1 will contain the block number and the diagonals [IMIN,JMIN,KMIN,IMAX,JMAX,KMAX]. Example: blk: 1 [168,0,0,268,100,0].
+            - **outer_faces_all** (List[Face]): This is a list of outer faces save as a list of Faces
+    """
+    gcd_array = list()
+    # Find the gcd of all the blocks 
+    for block_indx in range(len(blocks)):
+        block = blocks[block_indx]
+        gcd_array.append(math.gcd(block.IMAX-1, math.gcd(block.JMAX-1, block.KMAX-1)))
+    gcd_to_use = min(gcd_array) # You need to use the minimum gcd otherwise 1 block may not exactly match the next block. They all have to be scaled the same way.
+    blocks = reduce_blocks(deepcopy(blocks),gcd_to_use)
+
+    rotation_matrix = create_rotation_matrix(radians(rotation_angle),rotation_axis)
+    blocks_rotated = [rotate_block(b,rotation_matrix) for b in blocks] 
+   
+    # Check periodic within a block 
+    periodic_found = True
+    
+    # Here we make a list of all the outer faces
+    outer_faces_all = list() 
+    matched_faces_all = list()
+    periodic_faces = list()      # This is the output of the code 
+    periodic_faces_export = list() 
+    for o in outer_faces:
+        face = create_face(blocks[o['block_index']], int(o['IMIN']/gcd_to_use), int(o['IMAX']/gcd_to_use), 
+            int(o['JMIN']/gcd_to_use), int(o['JMAX']/gcd_to_use), int(o['KMIN']/gcd_to_use), int(o['KMAX']/gcd_to_use))
+        face.set_block_index(o['block_index'])
+        outer_faces_all.append(face)
+
+    for _,m in enumerate(matched_faces):
+        face1 = create_face(blocks_rotated[m['block1']['block_index']], int(m['block1']['IMIN']/gcd_to_use), int(m['block1']['IMAX']/gcd_to_use), 
+                            int(m['block1']['JMIN']/gcd_to_use), int(m['block1']['JMAX']/gcd_to_use), 
+                            int(m['block1']['KMIN']/gcd_to_use), int(m['block1']['KMAX']/gcd_to_use))
+        face2 = create_face(blocks[m['block2']['block_index']], int(m['block2']['IMIN']/gcd_to_use), int(m['block2']['IMAX']/gcd_to_use), 
+                            int(m['block2']['JMIN']/gcd_to_use), int(m['block2']['JMAX']/gcd_to_use), 
+                            int(m['block2']['KMIN']/gcd_to_use), int(m['block2']['KMAX']/gcd_to_use))
+        face1.set_block_index(m['block1']['block_index'])
+        face2.set_block_index(m['block2']['block_index'])
+        matched_faces_all.append(face1)
+        matched_faces_all.append(face2)
+
+    split_faces = list()         # List of split but free surfaces, this will be appended to outer_faces_to_remove list
+    while periodic_found:
+        periodic_found = False        
+        outer_faces_to_remove = list()  # Integer list of which outher surfaces to remove
+        outer_face_combos = list(combinations_with_replacement(range(len(outer_faces_all)),2))
+        t = trange(len(outer_face_combos))
+        for i in t: 
+            # Check if surfaces are periodic with each other
+            face1_indx = outer_face_combos[i][0]
+            face2_indx = outer_face_combos[i][1]
+            face1 = outer_faces_all[face1_indx]
+            face2 = outer_faces_all[face2_indx]
+            t.set_description(f"Checking connections block {face1.blockIndex} with {face2.blockIndex}")
+
+            if (face1.IMIN == face1.IMAX) and (face2.IMIN == face2.IMAX) or \
+                (face1.JMIN == face1.JMAX) and (face2.JMIN == face2.JMAX) or \
+                (face1.KMIN == face1.KMAX) and (face2.KMIN == face2.KMAX):
+                    
+                # Rotate Block 1 -> Check periodicity -> if not periodic -> Rotate Block 1 opposite direction -> Check periodicity
+                #   Rotate Block 1
+                block1_rotated = blocks_rotated[face1.blockIndex]
+                block2 = blocks[face2.blockIndex]
+                #   Check periodicity
+                df, periodic_faces_temp, split_faces_temp = __periodicity_check__(face1,face2,block1_rotated, block2)
+                
+                if len(periodic_faces_temp) > 0:
+                    outer_faces_to_remove.append(face1)
+                    outer_faces_to_remove.append(face2)
+                    outer_faces_to_remove.append(periodic_faces_temp[0])
+                    outer_faces_to_remove.append(periodic_faces_temp[1])
+                    periodic_faces.append(periodic_faces_temp)
+                    periodic_faces_export.append(face_matches_to_dict(periodic_faces_temp[0],periodic_faces_temp[1],block1_rotated,block2))
+                    split_faces.extend(split_faces_temp)
+                    periodic_found = True
+                    break
+
+        if (periodic_found):
+            outer_faces_to_remove = list(set(outer_faces_to_remove))
+            outer_faces_all = [p for p in outer_faces_all if p not in outer_faces_to_remove]
+            if len(split_faces)>0:
+                outer_faces_all.extend(split_faces)
+                split_faces.clear()
+
+    # This is an added check to make sure all periodic faces are in the outer_faces_to_remove
+    for p in periodic_faces:
+        outer_faces_to_remove.append(p[0])
+        outer_faces_to_remove.append(p[1])
+
+    for m in matched_faces_all:
+        outer_faces_to_remove.append(m)
+
+    outer_faces_to_remove = list(set(outer_faces_to_remove))    # Use only unique values
+    outer_faces_all = [p for p in outer_faces_all if p not in outer_faces_to_remove]    # remove from outer faces 
+    # remove any duplicate periodic face pairs 
+    indx_to_remove = list()
+    for i in range(len(periodic_faces)):
+        for j in range(i+1,len(periodic_faces)):
+            if periodic_faces[i][0] == periodic_faces[j][0]:
+                if periodic_faces[i][1] == periodic_faces[j][1]:
+                    indx_to_remove.append(j)
+            if periodic_faces[i][1] == periodic_faces[j][0]:
+                if periodic_faces[i][0] == periodic_faces[j][1]:
+                    indx_to_remove.append(j)
+    
+
+    periodic_faces_export = [periodic_faces_export[i] for i in range(len(periodic_faces)) if i not in indx_to_remove]
+    periodic_faces = [periodic_faces[i] for i in range(len(periodic_faces)) if i not in indx_to_remove]
+    # Export periodic faces and outer faces
+    outer_faces_export = list() 
+
+    for o in outer_faces_all:
+        outer_faces_export.append(o.to_dict())
+
+    # scale it up
+    for i in range(len(periodic_faces_export)):
+        periodic_faces_export[i]['block1']['IMIN'] *= gcd_to_use
+        periodic_faces_export[i]['block1']['JMIN'] *= gcd_to_use
+        periodic_faces_export[i]['block1']['KMIN'] *= gcd_to_use
+        periodic_faces_export[i]['block1']['IMAX'] *= gcd_to_use
+        periodic_faces_export[i]['block1']['JMAX'] *= gcd_to_use
+        periodic_faces_export[i]['block1']['KMAX'] *= gcd_to_use
+
+        if (periodic_faces_export[i]['block2']['IMIN'] == 168):
+            print("check")
+        periodic_faces_export[i]['block2']['IMIN'] *= gcd_to_use
+        periodic_faces_export[i]['block2']['JMIN'] *= gcd_to_use
+        periodic_faces_export[i]['block2']['KMIN'] *= gcd_to_use
+        periodic_faces_export[i]['block2']['IMAX'] *= gcd_to_use
+        periodic_faces_export[i]['block2']['JMAX'] *= gcd_to_use
+        periodic_faces_export[i]['block2']['KMAX'] *= gcd_to_use
+    
+    for i in range(len(periodic_faces)):
+        periodic_faces[i][0].I *= gcd_to_use
+        periodic_faces[i][0].J *= gcd_to_use
+        periodic_faces[i][0].K *= gcd_to_use
+
+        periodic_faces[i][1].I *= gcd_to_use
+        periodic_faces[i][1].J *= gcd_to_use
+        periodic_faces[i][1].K *= gcd_to_use
+
+    for j in range(len(outer_faces_export)):
+        outer_faces_export[j]['IMIN'] *= gcd_to_use
+        outer_faces_export[j]['JMIN'] *= gcd_to_use
+        outer_faces_export[j]['KMIN'] *= gcd_to_use
+        outer_faces_export[j]['IMAX'] *= gcd_to_use
+        outer_faces_export[j]['JMAX'] *= gcd_to_use
+        outer_faces_export[j]['KMAX'] *= gcd_to_use
+    
+    for j in range(len(outer_faces_all)):
+        outer_faces_all[j].I *= gcd_to_use
+        outer_faces_all[j].J *= gcd_to_use
+        outer_faces_all[j].K *= gcd_to_use
+    return periodic_faces_export, outer_faces_export, periodic_faces, outer_faces_all
 
 
 def linear_real_transform(face1:Face,face2:Face) -> Tuple:
@@ -443,12 +621,8 @@ def __periodicity_check__(face1:Face, face2:Face,block1:Block,block2:Block):
     Args:
         face1 (Face): An arbitrary face 
         face2 (Face): An arbitrary face 
-        blocks (List[Block]): List of all blocks
-        rotation_matrix (np.ndarray): rotation matrix
-        face1_indx (int): Index of face 1 inside the big array of faces. This is added to the list of faces to remove if periodicity is found
-        face2_indx (int): Index of face 1 inside the big array of faces. This is added to the list of faces to remove if periodicity is found
-        face1_block_indx (int): what block index face 1 is located in 
-        face2_block_indx (int): what block index face 2 is located in 
+        block1 (Block): block 1 cooresponding to face 1
+        block2 (Block): block 2 cooresponding to face 2 
 
     Returns:
         (tuple): containing
@@ -480,7 +654,7 @@ def __periodicity_check__(face1:Face, face2:Face,block1:Block,block2:Block):
 
         f2 = create_face(block2,imin=df['i2'].min(),jmin=df['j2'].min(),kmin=df['k2'].min(), imax=df['i2'].max(),jmax=df['j2'].max(),kmax=df['k2'].max())
         f2.set_block_index(face2.blockIndex)
-
+        
         split_faces.extend(split_face1)
         split_faces.extend(split_face2)
         if swapped:
